#! /usr/bin/env/python
# -*- coding: iso-8859-1 -*-

## Copyright 1999-2005 by LivingLogic AG, Bayreuth/Germany.
## Copyright 1999-2005 by Walter Dörwald
##
## All Rights Reserved
##
## See xist/__init__.py for the license

import py.test

from ll.xist import xsc
from ll.xist.ns import html, xml

import common


def test_walk_1():
	node = common.createfrag()
	def filter(*args):
		return (True, xsc.enterattrs, xsc.entercontent, True)

	def check(inmode, outmode):
		# call only for code coverage
		list(node.walk((True, xsc.entercontent, True), inmode=inmode, outmode=outmode))
		list(node.walk(filter, inmode=inmode, outmode=outmode))

	modes = (xsc.walknode, xsc.walkpath, xsc.walkindex, xsc.walkrootindex)
	for inmode in modes:
		for outmode in modes:
			yield check, inmode, outmode


def test_walk_2():
	def check(node, filter, result, inmode=xsc.walknode, outmode=xsc.walknode):
		def node2str(node):
			if isinstance(node, xsc.Node):
				if isinstance(node, xsc.Text):
					return "#"
				else:
					return node.xmlname
			else:
				return ".".join(map(node2str, node))
	
		assert map(node2str, node.walk(filter, inmode=inmode, outmode=outmode)) == result

	node = html.div(
		html.tr(
			html.th("gurk"),
			html.td("hurz"),
			id=html.b(42)
		),
		class_=html.i("hinz")
	)

	def filtertopdown(node):
		return (isinstance(node, xsc.Element), xsc.entercontent)
	def filterbottomup(node):
		return (xsc.entercontent, isinstance(node, xsc.Element))
	def filtertopdownattrs(node):
		return (isinstance(node, xsc.Element), xsc.enterattrs, xsc.entercontent)
	def filterbottomupattrs(node):
		return (xsc.enterattrs, xsc.entercontent, isinstance(node, xsc.Element))
	def filtertopdowntextonlyinattr(path):
		for node in path:
			if isinstance(node, xsc.Attr):
				inattr = True
				break
		else:
			inattr = False
		node = path[-1]
		if isinstance(node, xsc.Element):
			return (True, xsc.enterattrs, xsc.entercontent)
		if inattr and isinstance(node, xsc.Text):
			return (True, )
		else:
			return (xsc.entercontent, )

	def filtertopdownattrwithoutcontent(node):
		if isinstance(node, xsc.Element):
			return (True, xsc.entercontent, xsc.enterattrs)
		elif isinstance(node, (xsc.Attr, xsc.Text)):
			return (True, )
		else:
			return (xsc.entercontent, )

	yield check, node, filtertopdown, ["div", "tr", "th", "td"]
	yield check, node, filterbottomup, ["th", "td", "tr", "div"]
	yield check, node, filtertopdownattrs, ["div", "i", "tr", "b", "th", "td"]
	yield check, node, filtertopdownattrs, ["div", "div.class.i", "div.tr", "div.tr.id.b", "div.tr.th", "div.tr.td"], xsc.walknode, xsc.walkpath
	yield check, node, filterbottomupattrs, ["div.class.i", "div.tr.id.b", "div.tr.th", "div.tr.td", "div.tr", "div"], xsc.walknode, xsc.walkpath
	yield check, node, filtertopdowntextonlyinattr, ["div", "div.class.i", "div.class.i.#", "div.tr", "div.tr.id.b", "div.tr.id.b.#", "div.tr.th", "div.tr.td"], xsc.walkpath, xsc.walkpath
	yield check, node, filtertopdownattrwithoutcontent, ["div", "div.tr", "div.tr.th", "div.tr.th.#", "div.tr.td", "div.tr.td.#", "div.tr.id", "div.class"], xsc.walknode, xsc.walkpath


def test_walk_walkindex():
	e = html.div(
		"foo",
		html.a(
			"bar",
			xml.Attrs(lang="en"),
			href="baz",
		),
		"gurk",
	)
	res = list(e.walk(xsc.FindTypeAllAttrs(xsc.Text), outmode=xsc.walkindex))
	exp = [
		[0],
		[1, 0],
		[1, "href", 0],
		[1, (xml, "lang"), 0], # FIXME: This depends on dictionary iteration order
		[2]
	]
	assert res == exp


def test_walk_walkindexisnode():
	# Check that all walk modes return the same data
	def check(node):
		l1 = list(node.walk(xsc.FindTypeAllAttrs(xsc.Text), outmode=xsc.walknode))
		l2 = list(node.walk(xsc.FindTypeAllAttrs(xsc.Text), outmode=xsc.walkpath))
		l3 = list(node.walk(xsc.FindTypeAllAttrs(xsc.Text), outmode=xsc.walkindex))
		l4 = list(node.walk(xsc.FindTypeAllAttrs(xsc.Text), outmode=xsc.walkrootindex))
		assert len(l1) == len(l2) == len(l3) == len(l4)
		for (subnode, path, index, (root, rindex)) in zip(l1, l2, l3, l4):
			assert subnode is path[-1]
			assert subnode is node[index]
			assert subnode is root[rindex]

	for node in common.allnodes():
		yield check, node


def test_walkgetitem():
	e = html.div(
		1,
		html.div(
			2,
			html.div(
				3
			)
		)
	)
	isdiv = xsc.FindTypeAll(html.div)
	assert str(e.walk(isdiv)[0]) == "123"
	assert str(e.walk(isdiv)[-1]) == "3"
	py.test.raises(IndexError, e.walk(isdiv).__getitem__, 3)
	py.test.raises(IndexError, e.walk(isdiv).__getitem__, -4)



#! /usr/bin/env python
# -*- coding: iso-8859-1 -*-

from ll.xist import xsc, sims, parsers, xfind
from ll.xist.ns import xml, html, meta


url = "python-quotes.xml"


class note(xsc.Element):
	model = sims.NoElements()

	def convert(self, converter):
		e = html.div(self.content, class_="note")

		return e.convert(converter)


class foreign(xsc.Element):
	model = sims.NoElements()

	def convert(self, converter):
		e = html.em(self.content)

		return e.convert(converter)


class author(xsc.Element):
	model = sims.NoElements()

	def convert(self, converter):
		e = self.content

		return e.convert(converter)


class source(xsc.Element):
	model = sims.NoElements()

	def convert(self, converter):
		e = html.div(self.content, class_="source")

		return e.convert(converter)


class quotation(xsc.Element):
	model = sims.Elements(author, source, note)

	class Attrs(xsc.Element.Attrs):
		class date(xsc.TextAttr): pass

	def convert(self, converter):
		e = html.div(self.content, class_="quotation")

		return e.convert(converter)


class quotations(xsc.Element):
	model = sims.Elements(quotation)

	def convert(self, converter):
		e = xsc.Frag(
			html.DocTypeHTML401transitional(),
			html.html(
				html.head(
					meta.contenttype(),
					html.title("Python quotes"),
					meta.stylesheet(href="root:python-quotes.css")
				),
				html.body(
					html.h1("Python quotes"),
					html.div("(Generated from ", html.a(url, href=url), ")"),
					self/quotation # We want to get rid of the excessive whitespace
				)
			)
		)

		return e.convert(converter)


class __ns__(xsc.Namespace):
	xmlname = "quotations"
	xmlurl = "http://xmlns.livinglogic.de/xist/examples/python-quotes"
__ns__.update(vars())


if __name__ == "__main__":
	prefixes = xsc.Prefixes([__ns__, html, xml])
	base = "root:python-quotes.html"
	# We don't validate, because the XML contains broken HTML (<pre> inside <p>)
	e = parsers.parseURL(url, base=base, saxparser=parsers.ExpatParser, prefixes=prefixes, validate=False)
	e = xfind.first(e/quotations)
	e = e.compact().conv()
	e.write(open("python-quotes.html", "wb"), base=base, encoding="iso-8859-1", validate=False)



#! /usr/bin/env python
# -*- coding: iso-8859-1 -*-

## Copyright 1999-2005 by LivingLogic AG, Bayreuth/Germany.
## Copyright 1999-2005 by Walter Dörwald
##
## All Rights Reserved
##
## See xist/__init__.py for the license

"""
<par>A XSC module can be used with WebWare</par>
"""

__version__ = tuple(map(int, "$Revision: 2.8 $"[11:-2].split(".")))
# $Source: /data/cvsroot/LivingLogic/xist/_xist/wwi.py,v $

import HTTPServlet

import xsc

class Servlet(HTTPServlet.HTTPServlet):
	encoding = "utf-8"

	def respondToGet(self, trans):
		trans._response.write(self.content(trans).convert().asBytes(encoding=self.encoding))

	def content(self, trans):
		return xsc.Null



#! /usr/bin/env python
# -*- coding: iso-8859-1 -*-

# Setup script for XIST

__version__ = tuple(map(int,"$Revision: 2.64 $"[11:-2].split(".")))
# $Source: /data/cvsroot/LivingLogic/xist/setup.py,v $

from distutils.core import setup, Extension
import textwrap

DESCRIPTION = """
XIST is an extensible HTML/XML generator. XIST is also a DOM parser
(built on top of SAX2) with a very simple and pythonesque tree API.
Every XML element type corresponds to a Python class and these Python
classes provide a conversion method to transform the XML tree
(e.g. into HTML). XIST can be considered 'object oriented XSL'.
"""

CLASSIFIERS="""
Development Status :: 5 - Production/Stable
Environment :: Web Environment
Intended Audience :: Developers
License :: OSI Approved :: Python License (CNRI Python License)
Operating System :: OS Independent
Programming Language :: Python
Topic :: Internet :: WWW/HTTP :: Dynamic Content
Topic :: Internet :: WWW/HTTP :: Site Management
Topic :: Software Development :: Libraries :: Python Modules
Topic :: Text Processing :: Markup :: HTML
Topic :: Text Processing :: Markup :: XML
"""

KEYWORDS = """
XML
HTML
XHTML
XSLT
HSC
XSL-FO
SVG
WML
iHTML
"""

DESCRIPTION = "\n".join(textwrap.wrap(DESCRIPTION.strip(), width=64, replace_whitespace=True))

setup(
	name="ll-xist",
	version="2.10",
	description="An extensible HTML/XML generator",
	long_description=DESCRIPTION,
	author=u"Walter Dörwald".encode("utf-8"),
	author_email="walter@livinglogic.de",
	url="http://www.livinglogic.de/Python/xist/",
	download_url="http://www.livinglogic.de/Python/xist/Download.html",
	license="Python",
	classifiers=CLASSIFIERS.strip().splitlines(),
	keywords=",".join(KEYWORDS.strip().splitlines()),
	packages=["ll.xist", "ll.xist.ns"],
	package_dir={"ll.xist": "_xist"},
	ext_modules=[
		Extension("ll.xist.csstokenizer", ["_xist/csstokenizer.cxx"]),
		Extension("ll.xist.helpers", ["_xist/helpers.c"])
	],
	scripts=["scripts/dtd2xsc.py", "scripts/tld2xsc.py", "scripts/doc2txt.py" ]
)



#! /usr/bin/env python
# -*- coding: iso-8859-1 -*-

## Copyright 1999-2004 by LivingLogic AG, Bayreuth, Germany.
## Copyright 1999-2004 by Walter Dörwald
##
## All Rights Reserved
##
## See xist/__init__.py for the license

"""
<par>An &xist; namespace that contains definitions the
<link href="http://www.relaxng.org/">RELAX NG</link> definition.</par>
"""

__version__ = tuple(map(int, "$Revision: 2.63 $"[11:-2].split(".")))
# $Source: /data/cvsroot/LivingLogic/xist/_xist/ns/html.py,v $

from ll.xist import xsc, sims


class CommonAttrs(xsc.Element.Attrs):
	class ns(xsc.TextAttr): pass
	class datatypeLibrary(xsc.URLAttr): pass


class __ns__(xsc.Namespace):
	xmlname = "rng"
	xmlurl = "http://relaxng.org/ns/structure/1.0"
__ns__.makemod(vars())
